#!/usr/bin/tclsh

##############################################################################
#
# Auswertungstool fuer Atlantis Computer-Reports
# $ProjectVersion: 1.0.8 $   
# $ProjectDate: Tue, 03 Oct 2006 23:26:16 +0200 $
# Copyright: Norbert Donath   <Norbert.Donath@sigos.de>
#
#############################################################################

proc auswert {} {
  global argv regdb repFile mapDirName dbDirName version

  # $Format: "  set version \"$ProjectVersion: 1.12 $ ($ProjectDate: Tue, 18 Jan 2000 22:44:40 +0100 $)\""$
  set version "1.0.8 (Tue, 03 Oct 2006 23:26:16 +0200)"

  source parse.tcl
  source common.tcl
  ReadConfigFile

  puts "auswert.tcl $version"
  set numArgs [llength $argv]
  if {$numArgs != 2 && $numArgs != 3} {
    puts "Usage: auswert.tcl <report> <kartendir> <datadir>"
    puts "or"
    puts "auswert.tcl --init <datadir>"
    exit 1
  }

  if {$numArgs == 2} {
    if {[lindex $argv 0] != "--init"} {
      puts "Usage: auswert.tcl <report> <kartendir> <datadir>"
      puts "oder"
      puts "auswert.tcl --init <datadir>"
      exit 1
    }
    set dbDirName [lindex $argv 1]
    set dbDirName [string trimright $dbDirName /]
    if {[file isdirectory $dbDirName]} {
      puts "Benenne vorhandenes Datendirectory '$dbDirName' in '$dbDirName.old' um"
      if {[file isdirectory $dbDirName.old]} {
	puts "Directory '$dbDirName.old' existiert bereits"
	puts "Initialisierung kann nicht ausgefhrt werden"
	exit 1
      }
      if {[catch {file rename -force -- $dbDirName $dbDirName.old}]} {
	puts "Datendirectory konnte nicht umbenannt werden"
	exit 1
      }
    } elseif {[file exists $dbDirName]} {
      puts "'$dbDirName' existiert bereits, ist aber kein Directory"
      puts "Initialisierung kann nicht ausgefhrt werden"
      exit 1
    }
    puts "Lege Datendirectory '$dbDirName' an"
    if {[catch {file mkdir $dbDirName}]} {
      puts "Directory 'dbDirName' konnte nicht erzeugt werden"
      exit 1
    }
    puts "Initialisiere Regionendatenbank"
    InitRegDB $dbDirName
    exit 0
  }

  set repFileName [lindex $argv 0]
  set mapDirName [lindex $argv 1]
  set mapDirName [string trimright $mapDirName /]
  set dbDirName [lindex $argv 2]
  set dbDirName [string trimright $dbDirName /]

  puts "ffne Reportfile '$repFileName' zum Lesen"
  set repFile [FileOpenOrExit $repFileName "r"]

  if {[file exists $mapDirName] && ![file isdirectory $mapDirName]} {
    puts "'$mapDirName' existiert bereits ist aber kein Verzeichnis"
    exit 1
  } elseif {![file exists $mapDirName]} {
    puts "Lege Kartendirectory $mapDirName an"
    if {[catch {file mkdir $mapDirName}]} {
      puts "Directory '$mapDirName' konnte nicht angelegt werden"
      exit 1
    }
  }
  if {[file isdirectory $dbDirName]} {
    if {[file isdirectory $dbDirName.old]} {
      puts "Lsche vorhandene Sicherungskopie in '$dbDirName.old'"
      if {[catch {file delete -force -- $dbDirName.old}]} {
 	puts "Alte Sicherungskopie konnte nicht gelscht werden"
 	exit 1
      }
    }
    puts "Kopiere vorhandene Daten in '$dbDirName' nach '$dbDirName.old'"
    if {[catch {file copy -force -- $dbDirName $dbDirName.old}]} {
      puts "Datei konnte nicht kopiert werden!"
      exit 1
    }
  } else {
    if {[file exists $dbDirName] && ![file isdirectory $dbDirName]} {
      puts "$dbDirName existiert bereits, ist aber kein Verzeichnis"
      exit 1
    } else {
      puts "Lege Datendirectory '$dbDirName' an"
      if {[catch {file mkdir $dbDirName}]} {
	puts "Directory 'dbDirName' konnte nicht erzeugt werden"
	exit 1
      }
    }
  }

  if {![CheckReportFile [SkipToVersion]]} {
    puts "$repFileName enthlt keinen gltigen Computer Report"
    exit 1
  }
  ParseCR [GetMiscInfo]
  SetNameTables
  WriteMap
}


##############################################################################
# Name:            .
# Parameters:
# Return value:
# Description:
#############################################################################


##############################################################################
# Name:            InitRegDB
# Parameters:      dir: string
# Return value:    NONE
# Description:     Initialises the region data base and stores it to the file
#                  $dir/region.dat.
##############################################################################

proc InitRegDB {dir} {
  set fd [FileOpenOrExit "$dir/region.dat" "w+"]
  puts $fd "\# ATLANTIS_REGIONEN"
  puts $fd "set regList \{\}"
  close $fd
}


##############################################################################
# Name:            InitUnitDB
# Parameters:      dir: string
# Return value:    NONE
# Description:     Initialises the unit data base and stores it to the file
#                  $dir/units.dat.
##############################################################################

proc InitUnitDB {dir} {
  set fd [FileOpenOrExit "$dir/units.dat" "w+"]
  puts $fd "\# ATLANTIS_EINHEITEN"
  puts $fd "set unitList \{\}"
  close $fd
}


##############################################################################
# Name:            SaveUnits
# Parameters:      NONE
# Return value:    NONE
# Description:     Saves the unit list to a file.
#############################################################################

proc SaveUnits {} {
  global unitList dbDirName

  set fd [FileOpenOrExit "$dbDirName/units.dat" "w+"]
  puts $fd "\# ATLANTIS_EINHEITEN"
  puts $fd "set unitList \{\}"
  foreach tmpList $unitList {
    puts $fd "lappend unitList \{$tmpList\}"
  }
  close $fd
}


##############################################################################
# Name:            SaveRegions
# Parameters:      NONE
# Return value:    NONE
# Description:     Saves the region list to a file.
#############################################################################

proc SaveRegions {} {
  global regList dbDirName

  set fd [FileOpenOrExit "$dbDirName/region.dat" "w+"]
  puts $fd "\# ATLANTIS_REGIONEN"
  puts $fd "set regList \{\}"
  foreach {index tmpList} $regList {
    puts $fd "lappend regList \"$index\" \{$tmpList\}"
  }
  close $fd
}


##############################################################################
# Name:            TalentList
# Parameters:      line: string
# Return value:    ---: list
# Description:     Returns the talent values within line as a list.
#############################################################################

proc TalentList {line} {
  set tmp [TrimString $line]
  set pos [string first " " $tmp]
  set days [string range $tmp 0 [expr {$pos - 1}]]
  set level [string range $tmp [expr {$pos + 1}] end]
  return [list lerntage $days stufe $level]
}


##############################################################################
# Name:            RemoveOldUnits
# Parameters:      NONE
# Return value:    NONE
# Description:     Removes old units from the region data base.
#############################################################################

proc RemoveOldUnits {} {
  global regList miscList

  set round [GetValueFromList RUNDE $miscList]
  set newRegList {}
  foreach {id tmpList} $regList {
    if {$round > [GetValueFromList runde $tmpList]} {
      set index [expr {[lsearch -exact $tmpList "einheiten"] + 1}]
      set tmpList [lreplace $tmpList $index $index {}]
      set index [expr {[lsearch -exact $tmpList "burgen"] + 1}]
      set castles [GetValueFromList burgen $tmpList]
      set newCastles {}
      foreach castle $castles {
	set burgherrIndex [expr {[lsearch -exact $castle "burgherr"] + 1}]
	set castle [lreplace $castle $burgherrIndex $burgherrIndex ""]
	lappend newCastles $castle
      }
      set tmpList [lreplace $tmpList $index $index $newCastles]
    }
    lappend newRegList $id $tmpList
  }
  set regList $newRegList
}


##############################################################################
# Name:            RemoveOldShips
# Parameters:      NONE
# Return value:    NONE
# Description:     Removes old ships from the region data base.
#############################################################################

proc RemoveOldShips {} {
  global regList miscList

  set round [GetValueFromList RUNDE $miscList]
  set newRegList {}
  foreach {id tmpList} $regList {
    if {$round > [GetValueFromList runde $tmpList]} {
      set index [expr {[lsearch -exact $tmpList "schiffe"] + 1}]
      set tmpList [lreplace $tmpList $index $index {}]
    }
    lappend newRegList $id $tmpList
  }
  set regList $newRegList
}


##############################################################################
# Name:            WriteMap
# Parameters:      NONE
# Return value:    NONE
# Description:     Writes a HTML map into mapDirName
#############################################################################

proc WriteMap {} {
  global mapDirName areaData version libDir picArray

  puts "Kopiere Bilder nach $mapDirName"
  set types [list ebene ebene_schiff wald wald_schiff hochland hochland_schiff sumpf sumpf_schiff wueste wueste_schiff berge berge_schiff gletscher gletscher_schiff ozean ozean_schiff reg_unbek vollstaendig teilweise area_unbek]
  foreach type $types {
    if {[catch {file copy -force -- "$libDir/$picArray($type)" "$mapDirName"}]} {
      puts "Bild '$libDir/$picArray($type)' nicht gefunden"
      exit 1
    }
  }
  puts "Schreibe HTML-Karte..."
  set fd [FileOpenOrExit "$mapDirName/hauptkarte.html" w]
  GenerateAreaData
  WriteMiscFile
  WriteHTMLHeader $fd "Atlantis Hauptkarte"
  puts $fd "  <body bgcolor=\#bbbbbb text=\#000000>"
  puts $fd "    <h1>Atlantis Hauptkarte</h1>"
  puts $fd ""
  puts $fd "    <table border=0 cellspacing=0 cellpading=0>"
  puts $fd "      <tr>"
  puts $fd "        <td></td>"
  for {set i 0} {$i < 10} {incr i} {
    set xLow [expr {($i - 4) * 9}]
    puts $fd "        <td>$xLow</td>"
  }
  puts $fd "      </tr>"
  for {set i 0} {$i < 10} {incr i} {
    puts $fd "      <tr>"
    set yLow [expr {($i - 4) * 9}]
    puts $fd "        <td>$yLow</td>"
    for {set j 0} {$j < 10} {incr j} {
      if {$areaData($j,$i,count) == 0} {
	set imageName $picArray(area_unbek)
	set altName "Unbekanntes Gebiet"
      } elseif {$areaData($j,$i,count) < 45} {
	set imageName $picArray(teilweise)
	set altName "Teilweise bekannt"
      } else {
	set imageName $picArray(vollstaendig)
	set altName "Sehr gut bekannt"
      }
      GenerateAreaFile $j $i
      puts $fd "        <td><a href=\"$j-$i.html\"><img src=\"$imageName\" alt=\"$altName\" border=0 height=32 width=32></a>"
    }
    puts $fd "      </tr>"
  }
  puts $fd "    </table>"
  puts $fd "    <br>"
  puts $fd "    <a href=\"allgemeines.html\">Allgemeines</a>"
  WriteHTMLFooter $fd
  close $fd
}


##############################################################################
# Name:            WriteHTMLHeader
# Parameters:      fd: integer
#                  title: string
# Return value:    NONE
# Description:     Writes the header of a HTML file into file $fd. The title
#                  will be set to $title.
#############################################################################

proc WriteHTMLHeader {fd title} {
  global version

  puts $fd "<html>"
  puts $fd "  <head>"
  puts $fd "    <meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">"
  puts $fd "    <meta name=\"GENERATOR\" content=\"auswert.tcl $version\">"
  puts $fd "    <title>$title</title>"
  puts $fd "  </head>"
  puts $fd ""
}


##############################################################################
# Name:            WriteHTMLFooter
# Parameters:      fd: integer
# Return value:    NONE
# Description:     Writes a HTML footer into the file $fd.
#############################################################################

proc WriteHTMLFooter {fd} {
  global version

  puts $fd "    <br>"
  puts $fd "    <hr>"
  puts $fd "    Generated by auswert.tcl $version"
  puts $fd "  </body>"
  puts $fd "</html>"
}


##############################################################################
# Name:            GenerateAreaData
# Parameters:      NONE
# Return value:    NONE
# Description:     Sorts the known regions into a list for each 9x9 area.
#                  Determines the image shown for the area in hauptkarte.html.
#############################################################################

proc GenerateAreaData {} {
  global regList areaData

  for {set i 0} {$i < 10} {incr i} {
    for {set j 0} {$j < 10} {incr j} {
      set areaData($i,$j) {}
      set areaData($i,$j,count) 0
    }
  }

  foreach {index region} $regList {
    set x [GetValueFromList x $region]
    set y [GetValueFromList y $region]
    set areaCoords [GetAreaCoords $x $y]
    set tmp [GetAreaCoords [expr {$x - 1}] $y]
    if {$tmp != $areaCoords && [lsearch -exact $areaData($tmp) "$x,$y"] == -1} {
      lappend areaData($tmp) "$x,$y"
    }
    set tmp [GetAreaCoords [expr {$x + 1}] $y]
    if {$tmp != $areaCoords && [lsearch -exact $areaData($tmp) "$x,$y"] == -1} {
      lappend areaData($tmp) "$x,$y"
    }
    set tmp [GetAreaCoords $x [expr {$y - 1}]]
    if {$tmp != $areaCoords && [lsearch -exact $areaData($tmp) "$x,$y"] == -1} {
      lappend areaData($tmp) "$x,$y"
    }
    set tmp [GetAreaCoords $x [expr {$y + 1}]]
    if {$tmp != $areaCoords && [lsearch -exact $areaData($tmp) "$x,$y"] == -1} {
      lappend areaData($tmp) "$x,$y"
    }
    set tmp [GetAreaCoords [expr {$x - 1}] [expr {$y - 1}]]
    if {$tmp != $areaCoords && [lsearch -exact $areaData($tmp) "$x,$y"] == -1} {
      lappend areaData($tmp) "$x,$y"
    }
    set tmp [GetAreaCoords [expr {$x + 1}] [expr {$y - 1}]]
    if {$tmp != $areaCoords && [lsearch -exact $areaData($tmp) "$x,$y"] == -1} {
      lappend areaData($tmp) "$x,$y"
    }
    set tmp [GetAreaCoords [expr {$x + 1}] [expr {$y + 1}]]
    if {$tmp != $areaCoords && [lsearch -exact $areaData($tmp) "$x,$y"] == -1} {
      lappend areaData($tmp) "$x,$y"
    }
    set tmp [GetAreaCoords [expr {$x - 1}] [expr {$y + 1}]]
    if {$tmp != $areaCoords && [lsearch -exact $areaData($tmp) "$x,$y"] == -1} {
      lappend areaData($tmp) "$x,$y"
    }
    if {[lsearch -exact $areaData($areaCoords) "$x,$y"] == -1} {
      lappend areaData($areaCoords) "$x,$y"
    }
    incr areaData(${areaCoords},count)
  }
}


##############################################################################
# Name:            GetAreaCoords
# Parameters:      x: integer
#                  y: integer
# Return value:    ---: string
# Description:     Determines the area the region x,y belongs to. The area
#                  coordinations are returned in the format 'x,y'.
#                  Region coordinations are within a range of -36 an 53.
#                  Each area contains a block of 9x9 regions.
#############################################################################

proc GetAreaCoords {x y} {
  set xArea [expr {[NormalizeCoord $x] / 9 + 4}]
  set yArea [expr {[NormalizeCoord $y] / 9 + 4}]
  return "${xArea},${yArea}"
}


##############################################################################
# Name:            NormalizeCoord
# Parameters:      coord: integer
# Return value:    ---: string
# Description:     Returns a coordination within the range -36..53. Due to
#                  the geometry of the Atlantis world a value of -37 means 53
#                  and a value of 54 means -36.
#############################################################################

proc NormalizeCoord {coord} {
  if {$coord < -36} {
    return 53
  } elseif {$coord > 53} {
    return -36
  } else {
    return $coord
  }
}


##############################################################################
# Name:            GenerateAreaFile
# Parameters:      x: integer
#                  y: integer
# Return value:    NONE
# Description:     Writes a HTML file with the information of all known
#                  regions within this area.
#############################################################################

proc GenerateAreaFile {x y} {
  global regList areaData miscList mapDirName libDir picArray

  set xLow [expr {($x - 4) * 9}]
  set xHigh [expr {$xLow + 8}]
  set yLow [expr {($y - 4) * 9}]
  set yHigh [expr {$yLow + 8}]

  set fd [FileOpenOrExit "$mapDirName/$x-$y.html" w]
  WriteHTMLHeader $fd "Karte der Regionen von ($xLow,$yLow) bis ($xHigh,$yHigh)"
  puts $fd "  <body bgcolor=\#bbbbbb text=\#000000>"
  puts $fd "    <h1>Karte der Regionen von ($xLow,$yLow) bis ($xHigh,$yHigh)</h1>"
  puts $fd ""
  puts $fd "    <a name=\"karte\"></a>"
  puts $fd "    <table border=0 cellspacing=0 cellpading=0>"
  incr yLow -1
  incr yHigh 1
  incr xLow -1
  incr xHigh 1
  for {set i $yLow} {$i <= $yHigh} {incr i} {
    puts $fd "      <tr>"
    for {set j $xLow} {$j <= $xHigh} {incr j} {
      set ii [NormalizeCoord $i]
      set jj [NormalizeCoord $j]
      if {[lsearch -exact $areaData($x,$y) "$jj,$ii"] == -1} {
	set picName $picArray(reg_unbek)
	set regionName "Unbekannte Region"
	puts $fd "        <td><a href=\"\#unbekannt\"><img src=\"$picName\" alt=\"$regionName\" border=0 height=32 width=32></a></td>"
      } else {
	set region [GetValueFromList "$jj,$ii" $regList]
	set regionName [GetValueFromList name $region]
	if {[GetValueFromList schiffe $region] == {}} {
	  switch [GetValueFromList terrain $region] {
	    Ozean {set picName $picArray(ozean)}
	    Ebene {set picName $picArray(ebene)}
	    Wald {set picName $picArray(wald)}
	    Sumpf {set picName $picArray(sumpf)}
	    Berge {set picName $picArray(berge)}
	    Wueste {set picName $picArray(wueste)}
	    Hochland {set picName $picArray(hochland)}
	    Gletscher {set picName $picArray(gletscher)}
	  }
	} else {
	  switch [GetValueFromList terrain $region] {
	    Ozean {set picName $picArray(ozean_schiff)}
	    Ebene {set picName $picArray(ebene_schiff)}
	    Wald {set picName $picArray(wald_schiff)}
	    Sumpf {set picName $picArray(sumpf_schiff)}
	    Berge {set picName $picArray(berge_schiff)}
	    Wueste {set picName $picArray(wueste_schiff)}
	    Hochland {set picName $picArray(hochland_schiff)}
	    Gletscher {set picName $picArray(gletscher_schiff)}
	  }
	}
	puts $fd "        <td><a href=\"\#$jj-$ii\"><img src=\"$picName\" alt=\"$regionName\" border=0 height=32 width=32></a></td>"
      }
    }
    puts $fd "      </tr>"
  }
  puts $fd "    </table>"
  puts $fd "    <br>"
  puts $fd "    <a href=\"hauptkarte.html\">Hauptkarte</a><br>"
  puts $fd "    <hr>"
  for {set i $yLow} {$i <= $yHigh} {incr i} {
    for {set j $xLow} {$j <= $xHigh} {incr j} {
      set ii [NormalizeCoord $i]
      set jj [NormalizeCoord $j]
      set region [GetValueFromList "$jj,$ii" $regList]
      set title [GetRegionTitle $region]
      if {$title != "Unbekannte Region"} {
	puts $fd "    <a name=\"$jj-$ii\"><h2>$title</h2></a>"
	set uList [GetValueFromList einheiten $region]
	SortUnits $jj $ii $uList
	GetAmounts $uList
	WriteRegionDescription $fd $region
	WriteMarketPrices $fd $region
	WriteStatistics $fd $region
	WriteCastles $fd $region
	WriteShips $fd $region
	WriteUnits $fd $jj $ii
	WriteRound $fd $region
	puts $fd "    <a href=\"\#karte\">zurck zur Karte</a>"
	puts $fd "    <hr>"
      }
    }
  }
  puts $fd "    <a name=\"unbekannt\"><h2>Unbekannte Region</h2></a>"
  puts $fd "    <p>ber diese Region ist noch nichts bekannt.</p>"
  puts $fd "    <br>"
  puts $fd "    <a href=\"#karte\">zurck zur Karte</a>"
  puts $fd "    <hr>"

  WriteHTMLFooter $fd
  close $fd
}


##############################################################################
# Name:            WriteRegionDescription
# Parameters:      fd: int
#                  region: list
# Return value:    NONE
# Description:     Writes the region description into the file given in
#                  fd.
#############################################################################

proc WriteRegionDescription {fd region} {
  set name [EncodeHTMLSymbols [GetValueFromList name $region]]
  set x [GetValueFromList x $region]
  set y [GetValueFromList y $region]
  set terrain [GetValueFromList terrain $region]
  if {$terrain == ""} {
    puts $fd "    <p>ber diese Region ist noch nichts bekannt.</p>\n"
    return
  }
  set description "    <p>$name ($x,$y), $terrain"
  set tmp [GetValueFromList baeume $region]
  if {$tmp > 0} {
    append description ", $tmp Bume"
  }
  set tmp [GetValueFromList bauern $region]
  if {$tmp > 0} {
    append description ", $tmp Bauern"
  }
  set tmp [GetValueFromList silber $region]
  if {$tmp > 0} {
    append description ", $tmp\$"
  }
  set tmp [GetValueFromList pferde $region]
  if {$tmp > 0} {
    append description ", $tmp Pferde"
  }
  set tmp [GetValueFromList strasse $region]
  if {$tmp > 0} {
    append description ", das Straennetz ist zu $tmp% vollendet"
  }
  append description ".\n"
  foreach dir {nord sued ost west} {
    set tmp [GetValueFromList $dir $region]
    set terrain [lindex $tmp 0]
    switch $dir {
      nord {set dirStr "Norden"}
      sued {set dirStr "Sden"}
      west {set dirStr "Westen"}
      ost {set dirStr "Osten"}
    }
    if {$terrain == "Ozean"} {
      append description "      Im $dirStr: Ozean [lindex $tmp 1].\n"
    } elseif {$terrain == ""} {
      append description "<br>ber diese Region ist bisher sehr wenig bekannt\n"
      break
    } else {
      append description "      Im $dirStr: $terrain von [lindex $tmp 1].\n"
    }
  }
  set tmp [EncodeHTMLSymbols [GetValueFromList beschreibung $region]]
  append description "      $tmp\n"
  append description "    </p>"
  puts $fd $description
}


##############################################################################
# Name:            WriteMarketPrices
# Parameters:      fd: int
#                  region: list
# Return value:    NONE
# Description:     Writes the market prices into the file fd.
#############################################################################

proc WriteMarketPrices {fd region} {
  if {$region == {} || [GetValueFromList nord $region] == {{} {}} || [GetValueFromList terrain $region] == "Ozean"} {
    return
  }
  if {[GetValueFromList burgen $region] == {}} {
    puts $fd "    <p>Auf dem Markt wrden folgende Preise gelten:</p>"
  } else {
    puts $fd "    <p>Auf dem Markt gelten die folgenden Preise:</p>"
  }
  set goods [list balsam "Balsam" gewuerz "Gewrze" juwelen "Juwelen" myrrhe "Myrrhe" oel "l" seide "Seide" weihrauch "Weihrauch"]
  set prices [GetValueFromList preise $region]
  puts $fd "    <table border>"
  puts $fd "      <tr>"
  foreach {good name} $goods {
    puts $fd "        <td width=\"14%\" align=\"center\">$name</td>"
  }
  puts $fd "      </tr>"
  puts $fd "      <tr>"
  foreach {good name} $goods {
    set price [GetValueFromList $good $prices]
    if {$price < 0} {
      set soldGood $name
      set price [string range $price 1 end]
    }
    puts $fd "        <td align=\"center\">$price\$</td>"
  }
  puts $fd "      </tr>"
  puts $fd "    </table>"
  puts $fd "    <p>Die Bauern der Region produzieren $soldGood.</p>"
}


##############################################################################
# Name:            WriteStatistics
# Parameters:      fd: int
#                  region: list
# Return value:    NONE
# Description:     Writes the statistic data into the file fd.
#############################################################################

proc WriteStatistics {fd region} {
  global goodList goodNames amounts

  if {$region == {} || [GetValueFromList nord $region] == {{} {}} || [GetValueFromList terrain $region] == "Ozean"} {
    return
  }
  puts $fd "    <br>"
  puts $fd "    <table>"
  puts $fd "      <tr>"
  puts $fd "        <td>Lohn fr einen Monat Arbeiten:</td>"
  puts $fd "        <td>[GetValueFromList lohn $region]\$</td>"
  puts $fd "      </tr>\n      <tr>"
  set jobs [ComputeJobs $region]
  puts $fd "        <td>Arbeitspltze in der Region:</td>"
  puts $fd "        <td>max. $jobs</td>"
  puts $fd "      </tr>\n      <tr>"
  puts $fd "        <td>Erwarteter Lohn der arbeitenden Bauer:</td>"
  puts $fd "        <td>[GetValueFromList gesamtlohn $region]\$</td>"
  puts $fd "      </tr>\n      <tr>"
  puts $fd "        <td>Davon Geld fr Unterhaltung:</td>"
  puts $fd "        <td>[GetValueFromList unterhaltung $region]\$</td>"
  puts $fd "      </tr>\n      <tr>"
  puts $fd "        <td>Steuern fr eine stabile Population bei max. Unterhaltung:</td>"
  puts $fd "        <td>[GetValueFromList steuern $region]\$</td>"
  puts $fd "      </tr>\n      <tr>"
  puts $fd "        <td>Kaufkraft der Bauern bei max. Unterhaltung:</td>"
  puts $fd "        <td>[GetValueFromList kaufkraft $region]\$</td>"
  puts $fd "      </tr>\n      <tr>"
  set maxSell [GetValueFromList luxus $region]
  puts $fd "        <td>Anzahl Luxusgter zu kaufen bei stabilen Preisen:</td>"
  puts $fd "        <td>[expr {$maxSell * 6}]</td>"
  puts $fd "      </tr>\n      <tr>"
  puts $fd "        <td>Anzahl Luxusgter zu verkaufen bei stabilen Preisen:</td>"
  puts $fd "        <td>$maxSell</td>"
  puts $fd "      </tr>\n      <tr>"
  puts $fd "        <td>Anzahl rekrutierbarer Bauern:</td>"
  puts $fd "        <td>max. [GetValueFromList rekrutieren $region]</td>"
  foreach good $goodList {
    if {$amounts($good) > 0} {
      puts $fd "      </tr>\n      <tr>"
      puts $fd "        <td>$goodNames($good):</td>"
      puts $fd "        <td>$amounts($good)</td>"
    }
  }
  puts $fd "      </tr>\n      <tr>"
  puts $fd "        <td>Silber:</td>"
  puts $fd "        <td>$amounts(silber)</td>"
  puts $fd "      </tr>"
  puts $fd "    </table>\n    <br>"
  puts $fd "    <br>"
}


##############################################################################
# Name:            WriteCastles
# Parameters:      fd: int
#                  region: list
# Return value:    NONE
# Description:     Writes all castles of the area into the file fd.
#############################################################################

proc WriteCastles {fd region} {
  global unitList castleArray

  set castles [GetValueFromList burgen $region]
  if {$castles == {}} {
    return
  }
  foreach castle $castles {
    set description ""
    set name [EncodeHTMLSymbols [GetValueFromList name $castle]]
    set number [GetValueFromList nummer $castle]
    set size [GetValueFromList groesse $castle]
    set description "$name ($number), Gre $size, "
    append description [GetCastleType $size]
    set tmp [EncodeHTMLSymbols [GetValueFromList beschreibung $castle]]
    if {$tmp != ""} {
      append description ";\n$tmp"
    }
    set owner [GetValueFromList partei $castle]
    set colour [GetColour $owner]
    puts $fd "    <p><font color=\"$colour\">$description</font></p>"
    if {![info exists castleArray($number)]} {
      # no actual info available
      continue
    }
    set firstUnit [GetValueFromList burgherr $castle]
    PrintUnitDescription $fd $firstUnit
    foreach unit $castleArray($number) {
      if {$unit != $firstUnit} {
	PrintUnitDescription $fd $unit
      }
    }
    puts $fd "    <hr width=50%>"
    set castleArray($number) {}
  }
}


##############################################################################
# Name:            WriteShips
# Parameters:      fd: int
#                  region: list
# Return value:    NONE
# Description:     Writes all ships within the region into the file fd.
#############################################################################

proc WriteShips {fd region} {
  global unitList shipArray

  set ships [GetValueFromList schiffe $region]
  if {$ships == {}} {
    return
  }
  foreach ship $ships {
    set description ""
    set name [EncodeHTMLSymbols [GetValueFromList name $ship]]
    set number [GetValueFromList nummer $ship]
    set type [GetValueFromList typ $ship]
    set finished [GetValueFromList vollendet $ship]
    set description "$name ($number), $type"
    if {$finished != 100} {
      append description ", im Bau (${finished}% vollendet)"
    }
    append description ", [PrintShipLoad $ship]"
    set tmp [EncodeHTMLSymbols [GetValueFromList beschreibung $ship]]
    if {$tmp != ""} {
      append description ",\n $tmp"
    }
    set owner [GetValueFromList partei $ship]
    set colour [GetColour $owner]
    puts $fd "    <p><font color=\"$colour\">$description</font></p>"
    if {![info exists shipArray($number)]} {
      # no actual info available
      continue
    }
    set firstUnit [GetValueFromList kapitaen $ship]
    PrintUnitDescription $fd $firstUnit
    foreach unit $shipArray($number) {
      if {$unit != $firstUnit} {
	PrintUnitDescription $fd $unit
      }
    }
    set shipArray($number) {}
    puts $fd "    <hr width=50%>"
  }
}


##############################################################################
# Name:            WriteUnits
# Parameters:      fd: int
#                  x: int
#                  y: int
# Return value:    NONE
# Description:     Prints all units in region x,y which are not within a
#                  castle or a ship.
#############################################################################

proc WriteUnits {fd x y} {
  global regionArray

  if {[info exists regionArray($x,$y)]} {
    foreach unitNr $regionArray($x,$y) {
      PrintUnitDescription $fd $unitNr
    }
  }
  puts $fd "    <br>"
}


##############################################################################
# Name:            WriteRound
# Parameters:      fd: int
#                  region: list
# Return value:    NONE
# Description:     Writes the round the region data is from into the file fd.
#############################################################################

proc WriteRound {fd region} {
  global miscList monthNames

  if {$region != {}} {
    set round [GetValueFromList runde $region]
    set year [expr {$round / 12 + 1}]
    set monthNr [expr {$round % 12}]
    if {$monthNr == 0} {
      incr year -1
    }
    set month $monthNames($monthNr)
    puts $fd "    <p>Stand: $month $year</p>"
  }
}


##############################################################################
# Name:            GetColour
# Parameters:      party: int
# Return value:    ---: string
# Description:     Returns a colour string depending of the party number in
#                  party, the list of allied parties and the own party
#                  number.
#############################################################################

proc GetColour {party} {
  global miscList ownNumber alliedList

  if {$party == $ownNumber} {
    # own unit -> black
    return "\#000000"
  } elseif {[lsearch -exact $alliedList $party] != -1} {
    # allied unit -> dark green
    return "\#008000"
  } elseif {$party == -1} {
    # unknown -> dark blue
    return "\#000080"
  } else {
    # known but not own or allied -> dark red
    return "\#800000"
  }
}


##############################################################################
# Name:            PrintUnitDescription
# Parameters:      fd: int
#                  unitnr: int
# Return value:    NONE
# Description:     Prints the description of unit into file fd
#############################################################################

proc PrintUnitDescription {fd unitnr} {
  global unitList ownNumber talentNames goodList goodNames

  set unit [GetValueFromList $unitnr $unitList]
  set tmp [GetValueFromList parteiname $unit]
  set owner [GetValueFromList partei $unit]
  set colour [GetColour $owner]
  set initSymbol [GetInitSymbol $owner]
  set description "$initSymbol [EncodeHTMLSymbols [GetValueFromList name $unit]] ($unitnr), "
  if {$owner != -1} {
    append description "<b>$tmp ($owner)</b>, \n    "
  }
  set amount [GetValueFromList anzahl $unit]
  set tmp [GetValueFromList typ $unit]
  if {$amount == 1} {
    switch $tmp {
      Personen {set tmp "Person"}
    }
  }
  append description "$amount $tmp"
  set tmp [GetValueFromList bewacht $unit]
  if {$tmp} {
    append description ", <b>bewacht die Region</b>"
  }
  if {$owner == $ownNumber} {
    append description ", kmpft [GetValueFromList kaempft $unit]"
    append description ", [GetValueFromList silber $unit]\$"
    set talents [GetValueFromList talente $unit]
    append description ", [PrintUnitLoad $unitnr $unit],\n    "
    append description "Talente: "
    foreach {talent values} $talents {
      set days [GetValueFromList lerntage $values]
      if {$days > 0} {
	append description "$talentNames($talent) [GetValueFromList stufe $values] \[[expr {$days / $amount}]\],\n    "
      }
    }
    set spells [GetValueFromList sprueche $unit]
    if {$spells != {}} {
      append description "Zauber: "
      foreach spell $spells {
	append description "$spell, "
      }
      append description "\n    "
    }
    append description "Default: [GetValueFromList default $unit]\n    "
  }
  foreach good $goodList {
    set tmp [GetValueFromList $good $unit]
    if {$tmp > 0} {
      append description ", $goodNames($good): $tmp\n    "
    }
  }
  set tmp [GetValueFromList magisches $unit]
  if {$tmp != {}} {
    foreach magicItem $tmp {
      append description ", [lrange $magicItem 1 end]: [lindex $magicItem 0]"
    }
  }
  set tmp [EncodeHTMLSymbols [GetValueFromList beschreibung $unit]]
  if {$tmp != ""} {
    append description ", $tmp\n"
  }
  puts $fd "    <p><font color=\"$colour\">$description</font></p>"
}


##############################################################################
# Name:            PrintUnitLoad
# Parameters:      unitNumber: int
#                  unit: list
# Return value:    string
# Description:     Computes the load and the capacity of the unit and returns
#                  a string for the HTML file.
#############################################################################

proc PrintUnitLoad {unitNumber unit} {
  global capacity goodWeight

  set amount [GetValueFromList anzahl $unit]
  set horses [GetValueFromList pferd $unit]
  set carts [GetValueFromList wagen $unit]
  if {[GetValueFromList stufe [GetValueFromList reiten [GetValueFromList talente $unit]]] > 0 && $horses > 0} {
    set canRide 1
    set rideWeight [GetUnitWeight ride $unitNumber]
    if {$carts > [expr {2 * $horses}]} {
      set maxCarts [expr {$carts - $horses/2}]
      incr rideWeight [expr {($carts - $maxCarts) * $goodWeight(wagen)}]
    } else {
      set maxCarts $carts
    }
    set rideCapacity [expr {$maxCarts * $capacity(wagen) + ($horses - 2 * $maxCarts) * $capacity(pferd)}]
  } else {
    set canRide 0
    set rideCapacity 0
  }
  set footWeight [GetUnitWeight foot $unitNumber]
  set footCapacity [expr {$rideCapacity + $amount * $capacity(Personen)}]
  if {$canRide} {
    return "Kapazitt: [expr {$rideCapacity - $rideWeight}]/$rideCapacity, [expr {$footCapacity - $footWeight}]/$footCapacity frei"
  } else {
    return "Kapazitt: [expr {$footCapacity - $footWeight}]/$footCapacity frei"
  }
}


##############################################################################
# Name:            SetNameTables
# Parameters:      NONE
# Return value:    NONE
# Description:     Sets the global tables with talent and good names
#############################################################################

proc SetNameTables {} {
  global miscList talentNames goodNames ownNumber alliedList monthNames goodList goodWeight capacity

  set talentNames(armbrustschiessen) "Armbrustschieen"
  set talentNames(bogenschiessen) "Bogenschieen"
  set talentNames(katapultbedienung) "Katapultbedienung"
  set talentNames(schwertkampf) "Schwertkampf"
  set talentNames(speerkampf) "Speerkampf"
  set talentNames(reiten) "Reiten"
  set talentNames(taktik) "Taktik"
  set talentNames(bergbau) "Bergbau"
  set talentNames(burgenbau) "Burgenbau"
  set talentNames(handeln) "Handeln"
  set talentNames(holzfaellen) "Holzfllen"
  set talentNames(magie) "Magie"
  set talentNames(pferdedressur) "Pferdedressur"
  set talentNames(ruestungsbau) "Rstungsbau"
  set talentNames(schiffsbau) "Schiffsbau"
  set talentNames(segeln) "Segeln"
  set talentNames(steinbau) "Steinbau"
  set talentNames(strassenbau) "Straenbau"
  set talentNames(tarnung) "Tarnung"
  set talentNames(unterhaltung) "Unterhaltung"
  set talentNames(waffenbau) "Waffenbau"
  set talentNames(wagenbau) "Wagenbau"
  set talentNames(wahrnehmung) "Wahrnehmung"

  set goodList [list eisen holz stein pferd wagen katapult schwert speer armbrust bogen kettenhemd plattenpanzer balsam gewuerz juwel myrrhe oel seide weihrauch]

  set goodNames(eisen) "Eisenbarren"
  set goodNames(holz) "Holzstmme"
  set goodNames(stein) "Steinquader"
  set goodNames(pferd) "Pferde"
  set goodNames(wagen) "Wagen"
  set goodNames(katapult) "Katapulte"
  set goodNames(schwert) "Schwerter"
  set goodNames(speer) "Speere"
  set goodNames(armbrust) "Armbrste"
  set goodNames(bogen) "Bgen"
  set goodNames(kettenhemd) "Kettenhemden"
  set goodNames(plattenpanzer) "Plattenpanzer"
  set goodNames(balsam) "Balsamtpfe"
  set goodNames(gewuerz) "Gewrzballen"
  set goodNames(juwel) "Juwelen"
  set goodNames(myrrhe) "Myrrhesckchen"
  set goodNames(oel) "lamphoren"
  set goodNames(seide) "Seidenballen"
  set goodNames(weihrauch) "Weihrauchkisten"

  set goodWeight(Personen) 10
  set goodWeight(pferd) 50
  set goodWeight(wagen) 40
  set goodWeight(eisen) 5
  set goodWeight(holz) 5
  set goodWeight(stein) 60
  set goodWeight(schwert) 1
  set goodWeight(speer) 1
  set goodWeight(armbrust) 1
  set goodWeight(bogen) 1
  set goodWeight(katapult) 60
  set goodWeight(kettenhemd) 2
  set goodWeight(plattenpanzer) 4
  set goodWeight(balsam) 1
  set goodWeight(gewuerz) 1
  set goodWeight(juwel) 1
  set goodWeight(myrrhe) 1
  set goodWeight(oel) 1
  set goodWeight(seide) 1
  set goodWeight(weihrauch) 1

  set capacity(Personen) 5
  set capacity(pferd) 20
  set capacity(wagen) 140
  set capacity(Boot) 50
  set capacity(Langboot) 500
  set capacity(Drachenschiff) 1000
  set capacity(Karavelle) 3000
  set capacity(Trireme) 2000

  set ownNumber [GetValueFromList PARTEI $miscList]
  
  set alliedList {}
  foreach {number name} [GetValueFromList ALLIIERTE $miscList] {
    lappend alliedList $number
  }

  set monthNames(1) "Januar"
  set monthNames(2) "Februar"
  set monthNames(3) "Mrz"
  set monthNames(4) "April"
  set monthNames(5) "Mai"
  set monthNames(6) "Juni"
  set monthNames(7) "Juli"
  set monthNames(8) "August"
  set monthNames(9) "September"
  set monthNames(10) "Oktober"
  set monthNames(11) "November"
  set monthNames(0) "Dezember"
}


##############################################################################
# Name:            GetInitSymbol
# Parameters:      party: int
# Return value:    ---: char
# Description:     Return the appropriate first character for the unit
#                  description
#############################################################################

proc GetInitSymbol {party} {
  global ownNumber alliedList

  if {$party == $ownNumber} {
    return "*"
  }
  if {[lsearch $alliedList $party] != -1} {
    return "+"
  }
  if {$party == -1} {
    return "~"
  }
  return "-"
}


##############################################################################
# Name:            WriteMiscFile
# Parameters:      NONE
# Return value:    NONE
# Description:     Writes a HTML file with the misc information
#############################################################################

proc WriteMiscFile {} {
  global miscList mapDirName monthNames

  set fd [FileOpenOrExit "$mapDirName/allgemeines.html" w]
  WriteHTMLHeader $fd "Allgemeine Informationen"
  puts $fd "  <body bgcolor=\#bbbbbb text=\#000000>"
  puts $fd "    <a name=\"top\"><h1>Allgemeine Informationen</h1></a>"
  puts $fd ""
  set round [GetValueFromList RUNDE $miscList]
  set year [expr {$round / 12 + 1}]
  set monthNr [expr {$round % 12}]
  if {$monthNr == 0} {
    incr year -1
  }
  set month $monthNames($monthNr)
  puts $fd "    <p>Auswertung fr Partei [GetValueFromList PARTEI $miscList] im $month des Jahres $year.</p>"
  puts $fd "      <p>nchster ZAT: [GetValueFromList ZAT $miscList]</p>"
  puts $fd "    <br>"
  puts $fd "    <a href=\"\#fehler\">Fehler</a><br>"
  puts $fd "    <a href=\"\#warnungen\">Warnungen</a><br>"
  puts $fd "    <a href=\"\#meldungen\">Meldungen</a><br>"
  puts $fd "    <a href=\"\#kaempfe\">Kmpfe</a><br>"
  puts $fd "    <a href=\"\#ereignisse\">Ereignisse</a><br>"
  puts $fd "    <a href=\"\#einkommen\">Einkommen</a><br>"
  puts $fd "    <a href=\"\#handel\">Handel</a><br>"
  puts $fd "    <a href=\"\#produktion\">Produktion</a><br>"
  puts $fd "    <a href=\"\#bewegungen\">Bewegungen</a><br>"
  puts $fd "    <a href=\"\#zauber\">Zauber</a><br>"
  puts $fd "    <a href=\"\#alliierte\">Alliierte</a><br>"
  puts $fd "    <br>"
  puts $fd "    <h2>Fehler</h2>"
  set tmpList [GetValueFromList FEHLER $miscList]
  if {$tmpList == {}} {
    puts $fd "    <p>Es sind keine Fehler aufgetreten.</p>"
  } else {
    foreach msg $tmpList {
      puts $fd "    <p>$msg</p>"
    }
  }
  puts $fd "    <a href=\"\#top\">zurck</a><br>"
  puts $fd "    <a name=\"warnungen\"><h2>Warnungen</h2></a>"
  set tmpList [GetValueFromList WARNUNGEN $miscList]
  if {$tmpList == {}} {
    puts $fd "    <p>Es gibt keine Warnungen.</p>"
  } else {
    foreach msg $tmpList {
      puts $fd "    <p>$msg</p>"
    }
  }
  puts $fd "    <a href=\"\#top\">zurck</a><br>"
  puts $fd "    <a name=\"meldungen\"><h2>Meldungen</h2></a>"
  set tmpList [GetValueFromList MELDUNGEN $miscList]
  if {$tmpList == {}} {
    puts $fd "    <p>Es gibt keine neuen Meldungen.</p>"
  } else {
    foreach msg $tmpList {
      puts $fd "    <p>$msg</p>"
    }
  }
  puts $fd "    <a href=\"\#top\">zurck</a><br>"
  puts $fd "    <a name=\"kaempfe\"><h2>Kmpfe</h2></a>"
  set tmpList [GetValueFromList KAEMPFE $miscList]
  if {$tmpList == {}} {
    puts $fd "    <p>Es war alles friedlich.</p>"
  } else {
    foreach msg $tmpList {
      puts $fd "    <p>$msg</p>"
    }
  }
  puts $fd "    <a href=\"\#top\">zurck</a><br>"
  puts $fd "    <a name=\"ereignisse\"><h2>Ereignisse</h2></a>"
  set tmpList [GetValueFromList EREIGNISSE $miscList]
  if {$tmpList == {}} {
    puts $fd "    <p>Der letzte Monat war ziemlich langweilig; es gibt keine Ereignisse.</p>"
  } else {
    foreach msg $tmpList {
      puts $fd "    <p>$msg</p>"
    }
  }
  puts $fd "    <a href=\"\#top\">zurck</a><br>"
  puts $fd "    <a name=\"einkommen\"><h2>Einkommen</h2></a>"
  set tmpList [GetValueFromList EINKOMMEN $miscList]
  if {$tmpList == {}} {
    puts $fd "    <p>Wir haben im letzten Monat nichts verdient.</p>"
  } else {
    foreach msg $tmpList {
      puts $fd "    <p>$msg</p>"
    }
  }
  puts $fd "    <a href=\"\#top\">zurck</a><br>"
  puts $fd "    <a name=\"handel\"><h2>Handel</h2></a>"
  set tmpList [GetValueFromList HANDEL $miscList]
  if {$tmpList == {}} {
    puts $fd "    <p>Wir haben im letzten Monat nichts gehandelt.</p>"
  } else {
    foreach msg $tmpList {
      puts $fd "    <p>$msg</p>"
    }
  }
  puts $fd "    <a href=\"\#top\">zurck</a><br>"
  puts $fd "    <a name=\"produktion\"><h2>Produktion</h2></a>"
  set tmpList [GetValueFromList PRODUKTION $miscList]
  if {$tmpList == {}} {
    puts $fd "    <p>Wir haben im letzten Monat nichts produziert.</p>"
  } else {
    foreach msg $tmpList {
      puts $fd "    <p>$msg</p>"
    }
  }
  puts $fd "    <a href=\"\#top\">zurck</a><br>"
  puts $fd "    <a name=\"bewegungen\"><h2>Bewegungen</h2></a>"
  set tmpList [GetValueFromList BEWEGUNGEN $miscList]
  if {$tmpList == {}} {
    puts $fd "    <p>Niemand hat sich in eine andere Region begeben.</p>"
  } else {
    foreach msg $tmpList {
      puts $fd "    <p>$msg</p>"
    }
  }
  puts $fd "    <a href=\"\#top\">zurck</a><br>"
  puts $fd "    <a name=\"zauber\"><h2>Zauber</h2></a>"
  set tmpList [GetValueFromList ZAUBER $miscList]
  if {$tmpList == {}} {
    puts $fd "    <p>Es wurden keine neuen Zauber erforscht.</p>"
  } else {
    foreach msg $tmpList {
      puts $fd "    <p>$msg</p>"
    }
  }
  puts $fd "    <a href=\"\#top\">zurck</a><br>"
  puts $fd "    <a name=\"alliierte\"><h2>Alliierte</h2></a>"
  set tmpList [GetValueFromList ALLIIERTE $miscList]
  if {$tmpList == {}} {
    puts $fd "    <p>Wir helfen niemandem.</p>"
  } else {
    puts $fd "    <p>Wir helfen folgenden Parteien:</p>"
    puts $fd "    <table>"
    foreach {partyNr partyName} $tmpList {
      puts $fd "      <tr>"
      puts $fd "        <td>$partyNr</td>"
      puts $fd "        <td>$partyName</td>"
      puts $fd "      </tr>"
    }
    puts $fd "    </table>"
  }
  puts $fd "    <br>"
  puts $fd "    <a href=\"\#top\">zurck</a><br>"
  puts $fd "    <a href=\"hauptkarte.html\">Hauptkarte</a>"
  WriteHTMLFooter $fd
  close $fd
}


##############################################################################
# Name:            EncodeHTMLSymbols
# Parameters:      str: string
# Return value:    string
# Description:     Encodes the HTML symbols <, >, & and " in str
#############################################################################

proc EncodeHTMLSymbols {str} {
  if {[regexp {<|>|&|"} $str]} {
    regsub -all & $str {\&amp;} str
    regsub -all {"} $str {\&quot;} str
    regsub -all < $str {\&lt;} str
    regsub -all > $str {\&gt;} str
  }
  return $str
}



auswert
