
##############################################################################
#
# Auswertungstool fuer Atlantis Computer-Reports
# Routinen fuer den CR-Parser
# $ProjectVersion: 1.0.8 $   
# $ProjectDate: Tue, 03 Oct 2006 23:26:16 +0200 $
# Copyright: Norbert Donath   <Norbert.Donath@sigos.de>
#
#############################################################################


##############################################################################
# Name:            SkipToVersion
# Parameters:      NONE
# Return value:    verStr: string
# Description:     Reads preceding lines up to the version string from infile.
#                  Returns the line with the version string or "" if EOF is
#                  reached.
##############################################################################

proc SkipToVersion {} {
  global repFile

  set verStr ""
  set found 0
  while {!$found} {
    if {[gets $repFile verStr] == -1} {
      set found 1
      set verStr ""
    }
    if {[string first "Version des Computer Reports" $verStr] != -1} {
      set found 1
    }
  }
  return $verStr
}


##############################################################################
# Name:            CheckReportFile
# Parameters:      chkStr: string
# Return value:    ---: boolean
# Description:     Checks if the parameter chkStr identifies a valid Atlantis
#                  computer report and prints the version number.
##############################################################################

proc CheckReportFile {chkStr} {
  set index [string first ";" $chkStr]
  if {$index != -1} {
    incr index -1
    puts "Version des Computer Reports: [string range $chkStr 0 $index]"
    return 1
  } else {
    return 0
  }
}


##############################################################################
# Name:            GetMiscInfo
# Parameters:      NONE
# Return value:    tmpStr: string
# Description:     Reads miscellaneous information from the report and writes
#                  it in the file $dbDirName/misc.dat. File operation is
#                  omitted if the optional parameter withoutDB is set to 1.
##############################################################################

proc GetMiscInfo {{withoutDB 0}} {
  global dbDirName repFile miscList

  set miscList {}
  if {!$withoutDB} {
    set fd [FileOpenOrExit "$dbDirName/misc.dat" "w"]
    puts $fd "ATLANTIS_MISC"
  }

  if {[regsub -- "PARTEI " [ReadLineOrExit $repFile] "" trimmedStr] > 0} {
    lappend miscList PARTEI $trimmedStr
  } else {
    lappend miscList PARTEI ""
  }
  if {[regsub -- "; Runde" [ReadLineOrExit $repFile] "" trimmedStr] > 0} {
    lappend miscList RUNDE $trimmedStr
  } else {
    lappend miscList RUNDE ""
  }
  if {[regsub -- "; ZAT" [ReadLineOrExit $repFile] "" trimmedStr] > 0} {
    lappend miscList ZAT [TrimString $trimmedStr]
  } else {
    lappend miscList ZAT ""
  }
  if {[regsub -- "; Passwort" [ReadLineOrExit $repFile] "" trimmedStr] > 0} {
    lappend miscList Passwort [TrimString $trimmedStr]
  } else {
    lappend miscList Passwort ""
  }
  if {[ReadLineOrExit $repFile] != "FEHLER"} {
    lappend miscList FEHLER {}
  } else {
    set tmpList {}
    set tmpStr [ReadLineOrExit $repFile]
    while {$tmpStr != "WARNUNGEN"} {
      lappend tmpList [TrimString $tmpStr]
      set tmpStr [ReadLineOrExit $repFile]
    }
    lappend miscList FEHLER $tmpList
  }
  set tmpList {}
  set tmpStr [ReadLineOrExit $repFile]
  while {$tmpStr != "MELDUNGEN"} {
    lappend tmpList [TrimString $tmpStr]
    set tmpStr [ReadLineOrExit $repFile]
  }
  lappend miscList WARNUNGEN $tmpList
  set tmpList {}
  set tmpStr [ReadLineOrExit $repFile]
  while {$tmpStr != "KAEMPFE"} {
    lappend tmpList [TrimString $tmpStr]
    set tmpStr [ReadLineOrExit $repFile]
  }
  lappend miscList MELDUNGEN $tmpList
  set tmpList {}
  set tmpStr [ReadLineOrExit $repFile]
  while {$tmpStr != "EREIGNISSE"} {
    lappend tmpList [TrimString $tmpStr]
    set tmpStr [ReadLineOrExit $repFile]
  }
  lappend miscList KAEMPFE $tmpList
  set tmpList {}
  set tmpStr [ReadLineOrExit $repFile]
  while {$tmpStr != "EINKOMMEN"} {
    lappend tmpList [TrimString $tmpStr]
    set tmpStr [ReadLineOrExit $repFile]
  }
  lappend miscList EREIGNISSE $tmpList
  set tmpList {}
  set tmpStr [ReadLineOrExit $repFile]
  while {$tmpStr != "HANDEL"} {
    lappend tmpList [TrimString $tmpStr]
    set tmpStr [ReadLineOrExit $repFile]
  }
  lappend miscList EINKOMMEN $tmpList
  set tmpList {}
  set tmpStr [ReadLineOrExit $repFile]
  while {$tmpStr != "PRODUKTION"} {
    lappend tmpList [TrimString $tmpStr]
    set tmpStr [ReadLineOrExit $repFile]
  }
  lappend miscList HANDEL $tmpList
  set tmpList {}
  set tmpStr [ReadLineOrExit $repFile]
  while {$tmpStr != "BEWEGUNGEN"} {
    lappend tmpList [TrimString $tmpStr]
    set tmpStr [ReadLineOrExit $repFile]
  }
  lappend miscList PRODUKTION $tmpList
  set tmpList {}
  set tmpStr [ReadLineOrExit $repFile]
  while {$tmpStr != "ZAUBER"} {
    lappend tmpList [TrimString $tmpStr]
    set tmpStr [ReadLineOrExit $repFile]
  }
  lappend miscList BEWEGUNGEN $tmpList
  set tmpList {}
  set tmpStr [ReadLineOrExit $repFile]
  while {$tmpStr != "ALLIIERTE"} {
    lappend tmpList [TrimString $tmpStr]
    set tmpStr [ReadLineOrExit $repFile]
  }
  lappend miscList ZAUBER $tmpList
  set tmpList {}
  set tmpStr [ReadLineOrExit $repFile]
  while {![regexp -- REGION $tmpStr]} {
    lappend tmpList [TrimString $tmpStr]
    set tmpStr [ReadLineOrExit $repFile]
  }
  lappend miscList ALLIIERTE $tmpList

  if {!$withoutDB} {
    puts $fd "set miscList $miscList"
    close $fd
  }
  return $tmpStr
}


##############################################################################
# Name:            ParseCR
# Parameters:      firstLine: string
# Return value:    NONE
# Description:     Reads the region and unit data and stores it in the
#                  region database.
#############################################################################

proc ParseCR {firstLine {withoutDB 0}} {
  global dbDirName repFile regList miscList unitList

  if {!$withoutDB} {
    if {[file exists "$dbDirName/region.dat"]} {
      source "$dbDirName/region.dat"
    } else {
      puts "Regionendaten nicht gefunden"
      exit 1
    }
    if {![file exists "$dbDirName/region.dat"]} {
      puts "Fehler: Kann Regionendatenbank '$dbDirName/region.dat' nicht finden"
      exit 1
    } else {
      source $dbDirName/region.dat
    }
  }

  set init 1
  set unitList {}
  while {1} {
    if {$init} {
      set tmpStr $firstLine
      set init 0
    }
    if {[string first "REGION" $tmpStr] != -1} {
      set tmpList {}
      set x [GetCoords x $tmpStr]
      set y [GetCoords y $tmpStr]
      lappend tmpList x $x y $y
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList name $tmpStr
      puts "Lese Region $tmpStr ($x,$y)"
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList beschreibung $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList terrain $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList baeume $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList bauern $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList silber $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList pferde $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList strasse $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList lohn $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList gesamtlohn $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList unterhaltung $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList steuern $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList kaufkraft $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList luxus $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList rekrutieren $tmpStr
      set tmpList1 {}
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList1 $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList1 $tmpStr
      lappend tmpList nord $tmpList1
      set tmpList1 {}
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList1 $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList1 $tmpStr
      lappend tmpList sued $tmpList1
      set tmpList1 {}
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList1 $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList1 $tmpStr
      lappend tmpList ost $tmpList1
      set tmpList1 {}
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList1 $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList1 $tmpStr
      lappend tmpList west $tmpList1
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      if {$tmpStr != "PREISE"} {
	puts "Fehler im Report: 'PREISE' erwartet"
	exit 1
      }
      set tmpList1 {}
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList1 balsam $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList1 gewuerz $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList1 juwelen $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList1 myrrhe $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList1 oel $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList1 seide $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList1 weihrauch $tmpStr
      lappend tmpList preise $tmpList1
      set tmpList1 {}
      set tmpStr [GetCastle tmpList1]
      lappend tmpList burgen $tmpList1
      set tmpList1 {}
      set tmpStr [GetShip tmpList1 $tmpStr]
      lappend tmpList schiffe $tmpList1
      set tmpList1 {}
      set tmpStr [GetUnits tmpList1 $tmpStr]
      lappend tmpList einheiten $tmpList1
      lappend tmpList runde [GetValueFromList RUNDE $miscList]
      if {[RegionKnown $x $y]} {
	UpdateRegion $x $y $tmpList
      } else {
	NewRegion $x $y $tmpList
      }
      if {!$withoutDB} {
	CheckNeighbourRegions $x $y $tmpList
      }
      if {$tmpStr == ""} {
	# end of report
	break
      }
    } else {
      puts "Fehler im Report: 'REGION' erwartet"
      exit 1
    }
  }
  if {!$withoutDB} {
    SaveUnits
    RemoveOldUnits
    RemoveOldShips
    SaveRegions
  }
}


##############################################################################
# Name:            GetCoords
# Parameters:      type; String
#                  line; String
# Return value:    ---; int
# Description:     Returns the x or y coordination of the REGION tag
#############################################################################

proc GetCoords {type line} {
  set x ""
  set y ""
  regexp "REGION (-?\[0-9\]+) (-?\[0-9\]+)" $line dummy x y
  if {$type == "x"} {
    return $x
  } else {
    return $y
  }
}


##############################################################################
# Name:            GetNumber
# Parameters:      line; String
# Return value:    ---; int
# Description:     Returns the number after a tag.
#############################################################################

proc GetNumber {line} {
  set tmp [TrimString $line]
  set pos [string first " " $tmp]
  return [string range $tmp [expr {$pos + 1}] end]
}


##############################################################################
# Name:            TrimString
# Parameters:      str: String
# Return value:    ret: String
# Description:     Removes a ';' and all following characters from the
#                  input string. Also Removes all '"'.
#############################################################################

proc TrimString {str} {
  set ret ""
  if {[string first "\"" $str] != -1} {
    regexp -- "(.*\".*\"\[^;\]*);*.*" $str dummy ret
#    set ret [string trim $ret \"]
    set ret [string map {\" ""} $ret]
  } else {
    regexp -- "(\[^;\]*);*.*" $str dummy ret
  }
  return $ret
}


##############################################################################
# Name:            GetCastle
# Parameters:      retListName
# Return value:    ---: string
# Description:     Reads all castle information, if any. Returns the next
#                  line after the last castle definition or if no castle is
#                  defined the the first read line.
#############################################################################

proc GetCastle {retListName} {
  global repFile
  upvar 1 $retListName retList

  set retList {}
  while {1} {
    if {[gets $repFile tmpStr] == -1} {
      return ""
    }
    if {[string first "BURG" $tmpStr] == -1} {
      return $tmpStr
    }
    set tmpList [list nummer [GetNumber $tmpStr]]
    set tmpStr [TrimString [ReadLineOrExit $repFile]]
    lappend tmpList name $tmpStr
    set tmpStr [TrimString [ReadLineOrExit $repFile]]
    lappend tmpList beschreibung $tmpStr
    set tmpStr [TrimString [ReadLineOrExit $repFile]]
    lappend tmpList groesse $tmpStr
    set tmpStr [TrimString [ReadLineOrExit $repFile]]
    lappend tmpList burgherr $tmpStr
    set tmpStr [TrimString [ReadLineOrExit $repFile]]
    lappend tmpList partei $tmpStr
    lappend retList $tmpList
  }
}


##############################################################################
# Name:            GetShip
# Parameters:      retListName
#                  firstLine
# Return value:    ---: string
# Description:     Reads all ship information, if any. Returns the next
#                  line after the last ship definition or if no ship is
#                  defined the the first read line.
#############################################################################

proc GetShip {retListName firstLine} {
  global repFile
  upvar 1 $retListName retList

  set retList {}
  set tmpStr $firstLine
  while {1} {
    if {[string first "SCHIFF" $tmpStr] == -1} {
      return $tmpStr
    }
    set tmpList [list nummer [GetNumber $tmpStr]]
    set tmpStr [TrimString [ReadLineOrExit $repFile]]
    lappend tmpList name $tmpStr
    set tmpStr [TrimString [ReadLineOrExit $repFile]]
    lappend tmpList beschreibung $tmpStr
    set tmpStr [TrimString [ReadLineOrExit $repFile]]
    lappend tmpList typ $tmpStr
    set tmpStr [TrimString [ReadLineOrExit $repFile]]
    lappend tmpList vollendet $tmpStr
    set tmpStr [TrimString [ReadLineOrExit $repFile]]
    lappend tmpList kapitaen $tmpStr
    set tmpStr [TrimString [ReadLineOrExit $repFile]]
    lappend tmpList partei $tmpStr
    lappend retList $tmpList
    set tmpStr [TrimString [ReadLineOrExit $repFile]]
  }
}


##############################################################################
# Name:            GetUnits
# Parameters:      retListName: variable
#                  firstLine: string
# Return value:    tmpStr: string
# Description:     Reads all unit information in the current region. Returns
#                  the next REGION line or an empty string if it was the
#                  last region within the report.
#############################################################################

proc GetUnits {retListName firstLine} {
  global dbDirName repFile unitList miscList
  upvar 1 $retListName retList

  set retList {}
  set tmpStr $firstLine
  while {1} {
    set tmpList {}
    if {[string first "REGION" $tmpStr] != -1} {
      return $tmpStr
    }
    if {[string first "EINHEIT" $tmpStr] != -1} {
      set unitNumber [GetNumber $tmpStr]
      lappend retList $unitNumber
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList name $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList beschreibung $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList partei $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList parteiname $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList anzahl $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList typ $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList burgnr $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList schiffnr $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList bewacht $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      if {$tmpStr == "GEGENSTAENDE"} {
	# not a own unit
	lappend tmpList silber UNKNOWN
	lappend tmpList kaempft UNKNOWN
	lappend tmpList default UNKNOWN
	lappend tmpList talente {}
	lappend tmpList sprueche {UNKNOWN}
      } else {
	# own unit: much more information available
	lappend tmpList silber $tmpStr
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	switch -exact -- $tmpStr {
	  0 {set tmpStr vorne}
	  1 {set tmpStr hinten}
	  2 {set tmpStr nicht}
	  default {set tmpStr UNKNOWN}
	}
	lappend tmpList kaempft $tmpStr
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList default $tmpStr
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	if {$tmpStr != "TALENTE"} {
	  puts "Fehler im Report: 'TALENTE' erwartet"
	  exit 1
	}
	set tmpList1 {}
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 armbrustschiessen [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 bogenschiessen [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 katapultbedienung [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 schwertkampf [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 speerkampf [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 reiten [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 taktik [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 bergbau [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 burgenbau [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 handeln [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 holzfaellen [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 magie [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 pferdedressur [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 ruestungsbau [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 schiffsbau [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 segeln [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 steinbau [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 strassenbau [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 tarnung [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 unterhaltung [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 waffenbau [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 wagenbau [TalentList $tmpStr]
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	lappend tmpList1 wahrnehmung [TalentList $tmpStr]
	lappend tmpList talente $tmpList1
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	if {$tmpStr != "SPRUECHE"} {
	  puts "Fehler im Report: 'SPRUECHE' erwartet"
	  exit 1
	}
	set tmpStr [TrimString [ReadLineOrExit $repFile]]
	set tmpList1 {}
	while {$tmpStr != "GEGENSTAENDE"} {
	  lappend tmpList1 $tmpStr
	  set tmpStr [TrimString [ReadLineOrExit $repFile]]
	}
	lappend tmpList sprueche $tmpList1
      }
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList eisen $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList holz $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList stein $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList pferd $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList wagen $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList katapult $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList schwert $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList speer $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList armbrust $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList bogen $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList kettenhemd $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList plattenpanzer $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList balsam $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList gewuerz $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList juwel $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList myrrhe $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList oel $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList seide $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      lappend tmpList weihrauch $tmpStr
      set tmpStr [TrimString [ReadLineOrExit $repFile]]
      if {$tmpStr != "MAGISCHES"} {
	puts "Fehler im Report: 'MAGISCHES' erwartet"
	exit 1
      }
      set tmpList1 {}
      if {[gets $repFile tmpStr] == -1} {
	set tmpStr ""
      }
      while {[string first "REGION" $tmpStr] == -1 && [string first "EINHEIT" $tmpStr] == -1 && $tmpStr != ""} {
	lappend tmpList1 [TrimString $tmpStr]
	if {[gets $repFile tmpStr] == -1} {
	  set tmpStr ""
	}
      }
      lappend tmpList magisches $tmpList1
      lappend unitList $unitNumber $tmpList
      if {[string first "EINHEIT" $tmpStr] == -1} {
	# no more units in this region
	return $tmpStr
      }
    } else {
      puts "Fehler im Report: 'REGION' oder 'EINHEIT' erwartet"
      exit 1
    }
  }
}


##############################################################################
# Name:            RegionKnown
# Parameters:      x: integer
#                  y: integer
# Return value:    ---: boolean
# Description:     Checks if the region (x,y) is already in the regList.
#############################################################################

proc RegionKnown {x y} {
  global regList

  if {[lsearch -exact $regList "$x,$y"] != -1} {
    return 1
  } else {
    return 0
  }
}


##############################################################################
# Name:            CheckNeighbourRegions
# Parameters:      x: int
#                  y: int
#                  lst: list
# Return value:    NONE
# Description:     Checks if the neighbour regions are already known and
#                  if the terrain type has changed. Creates new regions if
#                  a neighbour region is unknown an changes the terrain
#                  type if it has changed.
#############################################################################

proc CheckNeighbourRegions {x y lst} {
  global regList

  foreach direction {nord sued ost west} {
    set xnew [GetNeighbourCoord x $direction $x]
    set ynew [GetNeighbourCoord y $direction $y]
    if {[RegionKnown $xnew $ynew]} {
      set storedRegion [GetValueFromList "$xnew,$ynew" $regList]
      set oldTerrain [GetValueFromList terrain $storedRegion]
      set newTerrain [lindex [GetValueFromList $direction $lst] 0]
      if {$oldTerrain != $newTerrain} {
	set index [lsearch -exact $storedRegion terrain]
	incr index
	set storedRegion [lreplace $storedRegion $index $index $newTerrain]
	UpdateRegion $xnew $ynew $storedRegion
      }
    } else {
      set tmpList [GetValueFromList $direction $lst]
      set name ""
      regexp -- "(.*) \\(.*\\)$" [lindex $tmpList 1] dummy name
      set newRegion [list "neighbour" [lindex $tmpList 0] $name]
      NewRegion $xnew $ynew $newRegion
    }
  }
}


##############################################################################
# Name:            GetNeighbourCoord
# Parameters:      what: char (x|y)
#                  dir: string (nord|sued|ost|west)
#                  coord: int
# Return value:    --: int
# Description:     Returns the coordination of the neighbour region in the
#                  given direction.
#############################################################################

proc GetNeighbourCoord {what dir coord} {
  switch $dir {
    nord {
      switch $what {
	x {}
	y {incr coord -1}
	default {return ERROR}
      }
    }
    sued {
      switch $what {
	x {}
	y {incr coord}
	default {return ERROR}
      }
    }
    ost {
      switch $what {
	x {incr coord}
	y {}
	default {return ERROR}
      }
    }
    west {
      switch $what {
	x {incr coord -1}
	y {}
	default {return ERROR}
      }
    }
    default {return ERROR}
  }
  if {$coord < -36} {
    set coord 53
  } elseif {$coord > 53} {
    set coord -36
  }
  return $coord
}


##############################################################################
# Name:            GetValueFromList
# Parameters:      what: string
#                  lst: list
# Return value:    ---: string
# Description:     Searches vor $what in $lst and returns the entry after
#                  $what or "" if it was not found.
#############################################################################

proc GetValueFromList {what lst} {
  set index [lsearch -exact $lst $what]
  if {$index == -1} {
    return ""
  } else {
    incr index
    return [lindex $lst $index]
  }
}


##############################################################################
# Name:            UpdateRegion
# Parameters:      x: integer
#                  y: integer
#                  lst: list
# Return value:    NONE
# Description:     Replaces the region entry in regList by the lst.
#############################################################################


proc UpdateRegion {x y lst} {
  global regList

  set index [lsearch -exact $regList "$x,$y"]
  incr index
  set regList [lreplace $regList $index $index $lst]
}


##############################################################################
# Name:            NewRegion
# Parameters:      x: integer
#                  y: integer
#                  lst: list
# Return value:    NONE
# Description:     Appends lst to the regList. If lst is empty a list with
#                  default values is created.
#############################################################################


proc NewRegion {x y lst} {
  global regList miscList

  if {$lst == {}} {
    set lst [list x $x y $y name "???" beschreibung "" terrain "???" baeume 0 bauern 0 silber 0 pferde 0 strasse 0 lohn 0 gesamtlohn 0 unterhaltung 0 steuern 0 kaufkraft 0 luxus 0 rekrutieren 0 nord [list "" ""] sued [list "" ""] ost [list "" ""] west [list "" ""] preise [list balsam 0 gewuerz 0 juwelen 0 myrrhe 0 oel 0 seide 0 weihrauch 0] einheiten {} burgen {} schiffe {} runde [GetValueFromList RUNDE $miscList]]
  } elseif {[lindex $lst 0] == "neighbour"} {
    set lst [list x $x y $y name [lindex $lst 2] beschreibung "" terrain [lindex $lst 1] baeume 0 bauern 0 silber 0 pferde 0 strasse 0 lohn 0 gesamtlohn 0 unterhaltung 0 steuern 0 kaufkraft 0 luxus 0 rekrutieren 0 nord [list "" ""] sued [list "" ""] ost [list "" ""] west [list "" ""] preise [list balsam 0 gewuerz 0 juwelen 0 myrrhe 0 oel 0 seide 0 weihrauch 0] einheiten {} burgen {} schiffe {} runde [GetValueFromList RUNDE $miscList]]
  }
  lappend regList "$x,$y" $lst
}


##############################################################################
# Name:            TalentList
# Parameters:      line: string
# Return value:    ---: list
# Description:     Returns the talent values within line as a list.
#############################################################################

proc TalentList {line} {
  set tmp [TrimString $line]
  set pos [string first " " $tmp]
  set days [string range $tmp 0 [expr {$pos - 1}]]
  set level [string range $tmp [expr {$pos + 1}] end]
  return [list lerntage $days stufe $level]
}


